#!/usr/bin/env python

import sys
import os
import django
from django.conf import settings
from django.test.utils import get_runner


APP_NAME = 'multilingual_flatpages'

if django.VERSION < (1, 10):
    settings.configure(
        DEBUG=True,
        DATABASES={
            'default': {
                'ENGINE': 'django.db.backends.sqlite3',
            }
        },
        USE_TZ=True,
        LANGUAGES=(
            ('es', 'Spanish'),
            ('en', 'English'),
        ),
        ROOT_URLCONF='{0}.urls'.format(APP_NAME),
        SITE_ID=1,
        INSTALLED_APPS=(
            'django.contrib.auth',
            'django.contrib.contenttypes',
            'django.contrib.sessions',
            'django.contrib.admin',
            'django.contrib.sites',
            'hvad',
            'tinymce',
            APP_NAME,
        ),
        TEMPLATE_CONTEXT_PROCESSORS=(
            "django.contrib.auth.context_processors.auth",
            "django.core.context_processors.debug",
            "django.core.context_processors.i18n",
            "django.core.context_processors.media",
            "django.core.context_processors.request",
        ),
        MIDDLEWARE_CLASSES=(
            'django.middleware.security.SecurityMiddleware',
            'django.contrib.sessions.middleware.SessionMiddleware',
            'django.middleware.common.CommonMiddleware',
            'django.middleware.csrf.CsrfViewMiddleware',
            'django.contrib.auth.middleware.AuthenticationMiddleware',
            'django.contrib.messages.middleware.MessageMiddleware',
            'django.middleware.clickjacking.XFrameOptionsMiddleware',
            'multilingual_flatpages.middleware.FlatpageFallbackMiddleware',
        ),
        TINYMCE_DEFAULT_CONFIG = {
            'language': 'en',
            'theme': 'modern',
            'height': 600,
            'plugins': [
                'advlist autolink lists link image charmap print preview anchor',
                'searchreplace visualblocks code fullscreen',
                'insertdatetime media table contextmenu paste',
            ],
            'toolbar': 'styleselect | bold italic | alignleft aligncenter alignright alignjustify | bullist numlist outdent indent | link image media | code preview',
            'menubar': False,
            'media_alt_source': False,
            'media_poster': False,
            'media_dimensions': False,
        }
    )
else:
    settings.configure(
        DEBUG=True,
        DATABASES={
            'default': {
                'ENGINE': 'django.db.backends.sqlite3',
            }
        },
        USE_TZ=True,
        LANGUAGES=(
            ('es', 'Spanish'),
            ('en', 'English'),
        ),
        ROOT_URLCONF='{0}.urls'.format(APP_NAME),
        SITE_ID=1,
        INSTALLED_APPS=(
            'django.contrib.auth',
            'django.contrib.contenttypes',
            'django.contrib.sessions',
            'django.contrib.admin',
            'django.contrib.sites',
            'hvad',
            'tinymce',
            APP_NAME,
        ),
        TEMPLATES=[
            {
                'BACKEND': 'django.template.backends.django.DjangoTemplates',
                'DIRS': [
                    # insert your TEMPLATE_DIRS here
                    os.path.join(os.path.dirname(__file__), "templates"),
                ],
                'OPTIONS': {
                    'context_processors': [
                        # Insert your TEMPLATE_CONTEXT_PROCESSORS here or use this
                        # list if you haven't customized them:
                        "django.contrib.auth.context_processors.auth",
                        "django.template.context_processors.debug",
                        "django.template.context_processors.i18n",
                        "django.template.context_processors.media",
                        "django.template.context_processors.request",
                    ],
                    'loaders': [
                        # insert your TEMPLATE_LOADERS here
                        'django.template.loaders.filesystem.Loader',
                        'django.template.loaders.app_directories.Loader',
                    ]
                },
            },
        ],
        MIDDLEWARE=[
            'django.middleware.security.SecurityMiddleware',
            'django.contrib.sessions.middleware.SessionMiddleware',
            'django.middleware.common.CommonMiddleware',
            'django.middleware.csrf.CsrfViewMiddleware',
            'django.contrib.auth.middleware.AuthenticationMiddleware',
            'django.contrib.messages.middleware.MessageMiddleware',
            'django.middleware.clickjacking.XFrameOptionsMiddleware',
            'multilingual_flatpages.middleware.FlatpageFallbackMiddleware',
        ],
        TINYMCE_DEFAULT_CONFIG={
            'language': 'en',
            'theme': 'modern',
            'height': 600,
            'plugins': [
                'advlist autolink lists link image charmap print preview anchor',
                'searchreplace visualblocks code fullscreen',
                'insertdatetime media table contextmenu paste',
            ],
            'toolbar': 'styleselect | bold italic | alignleft aligncenter alignright alignjustify | bullist numlist outdent indent | link image media | code preview',
            'menubar': False,
            'media_alt_source': False,
            'media_poster': False,
            'media_dimensions': False,
        }
    )

if hasattr(django, 'setup'):
    django.setup()
TestRunner = get_runner(settings)
test_runner = TestRunner()
failures = test_runner.run_tests([APP_NAME])
if failures:
    sys.exit(failures)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-09-21 14:05
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('multilingual_flatpages', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='flatpage',
            name='name',
            field=models.CharField(default='', max_length=100, verbose_name='Name'),
            preserve_default=False,
        ),
    ]



from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class FlatPagesConfig(AppConfig):
    name = 'multilingual_flatpages'
    verbose_name = _("Multilingual Flat Pages")



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-11-29 09:53
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('multilingual_flatpages', '0002_flatpage_name'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='flatpage',
            options={'ordering': ('slug',), 'verbose_name': 'flat page', 'verbose_name_plural': 'flat pages'},
        ),
        migrations.RemoveField(
            model_name='flatpagetranslation',
            name='url',
        ),
        migrations.AddField(
            model_name='flatpagetranslation',
            name='slug',
            field=models.CharField(db_index=True, default='', max_length=100, verbose_name='slug'),
            preserve_default=False,
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-09-21 10:59
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
    ]

    operations = [
        migrations.CreateModel(
            name='FlatPage',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('enable_comments', models.BooleanField(default=False, verbose_name='enable comments')),
                ('template_name', models.CharField(blank=True, help_text="Example: 'flatpages/contact_page.html'. If this isn't provided, the system will use 'flatpages/default.html'.", max_length=70, verbose_name='template name')),
                ('registration_required', models.BooleanField(default=False, help_text='If this is checked, only logged-in users will be able to view the page.', verbose_name='registration required')),
                ('sites', models.ManyToManyField(to='sites.Site', verbose_name='sites')),
            ],
            options={
                'ordering': ('url',),
                'verbose_name': 'flat page',
                'verbose_name_plural': 'flat pages',
            },
        ),
        migrations.CreateModel(
            name='FlatPageTranslation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('url', models.CharField(db_index=True, max_length=100, verbose_name='URL')),
                ('title', models.CharField(max_length=200, verbose_name='title')),
                ('content', models.TextField(blank=True, verbose_name='content')),
                ('language_code', models.CharField(db_index=True, max_length=15)),
                ('master', models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='multilingual_flatpages.FlatPage')),
            ],
            options={
                'managed': True,
                'abstract': False,
                'db_table': 'multilingual_flatpages_flatpage_translation',
                'db_tablespace': '',
                'default_permissions': (),
            },
        ),
        migrations.AlterUniqueTogether(
            name='flatpagetranslation',
            unique_together=set([('language_code', 'master')]),
        ),
    ]



from __future__ import unicode_literals
from django.contrib.sites.models import Site
from django.core.urlresolvers import reverse
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import activate
from django.utils.translation import get_language
from django.utils.translation import ugettext_lazy as _
from hvad.models import TranslatableModel, TranslatedFields


@python_2_unicode_compatible
class FlatPage(TranslatableModel):
    name = models.CharField(_('Name'), max_length=100)
    translations = TranslatedFields(
        slug=models.CharField(_('slug'), max_length=100, db_index=True),
        title=models.CharField(_('title'), max_length=200),
        content=models.TextField(_('content'), blank=True),
    )
    enable_comments = models.BooleanField(_('enable comments'), default=False)
    template_name = models.CharField(
        _('template name'),
        max_length=70,
        blank=True,
        help_text=_(
            "Example: 'multilingual_flatpages/contact_page.html'. If this isn't provided, "
            "the system will use 'multilingual_flatpages/default.html'."
        ),
    )
    registration_required = models.BooleanField(
        _('registration required'),
        help_text=_("If this is checked, only logged-in users will be able to view the page."),
        default=False,
    )
    sites = models.ManyToManyField(Site, verbose_name=_('sites'))

    class Meta:
        app_label = "multilingual_flatpages"
        verbose_name = _('flat page')
        verbose_name_plural = _('flat pages')
        ordering = ('slug',)

    def __str__(self):
        return "%s -- %s" % (self.slug, self.title)

    def get_absolute_url(self):
        # Handle script prefix manually because we bypass reverse()
        if self.slug.startswith("/"):
            slug = self.slug[1:]
        else:
            slug = self.slug
        return reverse('multilingual_flatpages', args=[slug])

    def get_translation_url(self, lang):
        old_lang = get_language()
        activate(lang)
        translation = FlatPage.objects.language(lang).get(id=self.id)
        url = translation.get_absolute_url()
        activate(old_lang)
        return url



from django.contrib import admin
from django.utils.translation import ugettext_lazy as _
from hvad.admin import TranslatableAdmin
from multilingual_flatpages.forms import FlatpageForm
from multilingual_flatpages.models import FlatPage


@admin.register(FlatPage)
class MultiFlatPageAdmin(TranslatableAdmin):

    def get_title(self, obj):
        return obj.safe_translation_getter('title')
    get_title.short_description = _('Title')

    def get_slug(self, obj):
        return obj.safe_translation_getter('slug')
    get_slug.short_description = _('slug')

    form = FlatpageForm
    use_fieldsets = (
        (None, {'fields': ('name', 'slug', 'title', 'content', 'sites')}),
        (_('Advanced options'), {
            'classes': ('collapse',),
            'fields': ('registration_required', 'template_name'),
        }),
    )
    list_display = ('name', 'get_slug', 'get_title', 'all_translations')
    list_filter = ('sites', 'registration_required')
    # search_fields = ('url', 'title')

    def get_fieldsets(self, request, obj=None):
        return self.use_fieldsets



from django.conf import settings
from django.http import Http404
from multilingual_flatpages.views import flatpage

try:
    from django.utils.deprecation import MiddlewareMixin as Base
except ImportError:
    Base = object


class FlatpageFallbackMiddleware(Base):

    def process_response(self, request, response):
        if response.status_code != 404:
            # No need to check for a flatpage for non-404 responses.
            return response
        try:
            return flatpage(request, request.path_info)
        # Return the original response if any errors happened. Because this
        # is a middleware, we can't assume the errors will be caught elsewhere.
        except Http404:
            return response
        except Exception:
            if settings.DEBUG:
                raise
            return response



# -*- coding: utf-8 -*-
# /usr/bin/env python

import uuid
from setuptools import setup, find_packages
try: # for pip >= 10
    from pip._internal.req import parse_requirements
except ImportError: # for pip <= 9.0.3
    from pip.req import parse_requirements

def get_requirements(source):
    try:
        install_reqs = parse_requirements(source, session=uuid.uuid1())
    except TypeError:
        # Older version of pip.
        install_reqs = parse_requirements(source)
    try:
        required = [str(ir.req) for ir in install_reqs]
    except:
        required = [str(ir.requirement) for ir in install_reqs]
    return list(required)

version = '0.8.4'

setup(name='multilingual_flatpages',
      version=version,
      description="""A flatpage is a simple object with a URL, title and content. Use it for one-off, special-case pages, such as “About” or “Privacy Policy” pages, that you want to store in a database but for which you don’t want to develop a custom Django application. A flatpage can use a custom template or a default, systemwide flatpage template. It can be associated with one, or multiple, sites. This version is a fork of django.contrib.flatpages package made it multilingual.""",
      long_description=open("README.rst").read() + "\n" +
                       open("CHANGELOG.txt").read(),
      classifiers=[],
      keywords='',
      author="Urtzi Odriozola (CodeSyntax http://codesyntax.com)",
      author_email="uodriozola@codesyntax.com",
      url="https://github.com/codesyntax/django-multilingual-flatpages",
      packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
      include_package_data=True,
      zip_safe=False,
      install_requires=get_requirements('requirements.txt'),
)



from django import forms
from django.conf import settings
from django.utils.translation import ugettext, ugettext_lazy as _
from hvad.forms import TranslatableModelForm
from multilingual_flatpages.models import FlatPage
from tinymce.widgets import TinyMCE


class FlatpageForm(TranslatableModelForm):
    slug = forms.RegexField(
        label=_("slug"),
        max_length=100,
        regex=r'^[-\w/\.~]+$',
        help_text=_("Example: '/about/contact/'. Make sure to have leading and trailing slashes."),
        error_messages={
            "invalid": _(
                "This value must contain only letters, numbers, dots, "
                "underscores, dashes, slashes or tildes."
            ),
        },
    )
    content = forms.CharField(widget=TinyMCE(attrs={'cols': 80, 'rows': 30}, mce_attrs=settings.TINYMCE_DEFAULT_CONFIG))

    class Meta:
        model = FlatPage
        # fields = '__all__'

    def clean_url(self):
        slug = self.cleaned_data['slug']
        if not slug.startswith('/'):
            raise forms.ValidationError(
                ugettext("URL is missing a leading slash."),
                code='missing_leading_slash',
            )
        if (settings.APPEND_SLASH and (
                'django.middleware.common.CommonMiddleware' in settings.MIDDLEWARE_CLASSES) and
                not slug.endswith('/')):
            raise forms.ValidationError(
                ugettext("URL is missing a trailing slash."),
                code='missing_trailing_slash',
            )
        return slug

    def clean(self):
        slug = self.cleaned_data.get('slug')
        sites = self.cleaned_data.get('sites')

        same_url = FlatPage.objects.language().filter(slug=slug)
        if self.instance.pk:
            same_url = same_url.exclude(pk=self.instance.pk)

        if sites and same_url.filter(sites__in=sites).exists():
            for site in sites:
                if same_url.filter(sites=site).exists():
                    raise forms.ValidationError(
                        _('Flatpage with url %(url)s already exists for site %(site)s'),
                        code='duplicate_url',
                        params={'slug': slug, 'site': site},
                    )

        return super(FlatpageForm, self).clean()



from django.contrib.sites.models import Site
from django.test import TestCase
from django.utils.translation import activate

try:
    from django.core.urlresolvers import reverse
except ImportError:  # django < 1.10
    from django.urls import reverse
from django.test import Client
from multilingual_flatpages.models import FlatPage
try:
    from django.contrib.auth import get_user_model
    User = get_user_model()
except ImportError:  # django < 1.5
    from django.contrib.auth.models import User


class BasicTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(
            'john',
            'lennon@thebeatles.com',
            'johnpassword')
        self.user.save()
        self.flatpage = FlatPage.objects.language('en').create(
            slug='/en-slug/',
            title='EN Title',
            content='EN Content',
        )
        self.flatpage.sites.add(Site.objects.get(id='1'))
        self.flatpage.save()

    def test_flatpage_view(self):
        activate('en')
        c = Client()
        url = reverse('multilingual_flatpages', args=[self.flatpage.slug])
        response = c.get(url)
        self.assertEqual(response.status_code, 200)
